#!/usr/bin/env Rscript

# Author: Lukas

#' This script extracts Nationality from physicist texts
#' Try to search text for first occurrence of an NORP entity
#'
#' @export
#' @param article Article in HTML-format
#' @param annotations CNLP Annotations
#' @return String with nationality of the physicist|NA
get_nationality <- function(article, annotations) {
  
  # Try to extract nationality from text as first NORP entity sentence
  # - Get entities
  # - Get NORPs
  # - Get first NORP
  # - Return NA if length < 4 because some empty strings and garbage would be returned else
  entities_obj <- cnlp_get_entity(annotations)
  norps <- entities_obj[entities_obj$entity_type == "NORP",]
  nationality <- norps$entity[1]
  ifelse(nchar(nationality) >= 4, nationality, NA)
}
